@@ -30,7 +30,7 @@
 # The fact that you are presently reading this means that you have had
 # knowledge of the CeCILL-C license and that you accept its terms.
 #
-# $Id: Ssh.py 243 2010-03-05 14:31:41Z st-cea $
+# $Id: Ssh.py 279 2010-06-16 15:29:15Z st-cea $
 
 """
 ClusterShell Ssh/Scp support
@@ -177,8 +177,6 @@ class Scp(Ssh):
         self.source = source
         self.dest = dest
         self.popen = None
-        self.file_reader = None
-        self.file_writer = None
 
         # Directory check
         self.isdir = os.path.isdir(self.source)
@@ -234,6 +232,7 @@ class Scp(Ssh):
 
         self.popen = self._exec_nonblock(cmd_l)
         self.file_reader = self.popen.stdout
+        self.file_error = self.popen.stderr
         self.file_writer = self.popen.stdin
 
         return self
