@@ -31,7 +31,7 @@
 # The fact that you are presently reading this means that you have had
 # knowledge of the CeCILL-C license and that you accept its terms.
 #
-# $Id: nodeset.py 266 2010-05-25 07:50:55Z st-cea $
+# $Id: nodeset.py 280 2010-06-16 21:15:41Z st-cea $
 
 """
 Usage: nodeset [COMMAND] [OPTIONS] [ns1 [-ixX] ns2|...]
@@ -92,7 +92,8 @@ from ClusterShell.NodeUtils import GroupResolverSourceError
 from ClusterShell.NodeUtils import GroupSourceException
 from ClusterShell.NodeUtils import GroupSourceNoUpcall
 try:
-    from ClusterShell.NodeSet import NodeSet, NodeSetParseError
+    from ClusterShell.NodeSet import NodeSet
+    from ClusterShell.NodeSet import NodeSetExternalError, NodeSetParseError
     from ClusterShell.NodeSet import RangeSet, RangeSetParseError
     from ClusterShell.NodeSet import grouplist, STD_GROUP_RESOLVER
     from ClusterShell import __version__
@@ -299,6 +300,9 @@ if __name__ == '__main__':
     except SyntaxError:
         print >> sys.stderr, "ERROR: invalid separator"
         sys.exit(1)
+    except NodeSetExternalError, e:
+        print >> sys.stderr, "ERROR: external error:", e
+        sys.exit(1)
     except GroupResolverSourceError, e:
         print >> sys.stderr, "ERROR: unknown group source: \"%s\"" % e
         sys.exit(1)
