@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # ClusterShell.Node* test suite
 # Written by S. Thiell 2010-03-18
-# $Id: NodeSetGroupTest.py 261 2010-05-24 21:59:56Z st-cea $
+# $Id: NodeSetGroupTest.py 280 2010-06-16 21:15:41Z st-cea $
 
 
 """Unit test for NodeSet with Group support"""
@@ -185,8 +185,7 @@ class NodeSetGroupTest(unittest.TestCase):
         self.assertEqual(nodeset, NodeSet("montana[38-41]"))
         self.assertEqual(str(nodeset), "montana[38-41]")
 
-        NodeSet.fromall(resolver=res)
-        #self.assertRaises(NodeSetExternalError, NodeSet.fromall)
+        self.assertRaises(NodeSetExternalError, NodeSet.fromall, resolver=res)
 
     
     def testConfigEmpty(self):
